from netgen.meshing import *
from ngsolve import *

SB = 20.0       # specimen width
CA = 0.5*SB     # crack length
SH = 0.5*SB     # specimen half-height

mesh=Mesh("SEC_2D_lin_tri.vol")

# Define element order
ele_order = 2

# Perform nref uniform refinements
# nref = 4
# for inum in range(nref):
  # mesh.Refine()

##   Elasticity parameters
E = 207.0e3 # Elastic modulus
nu = 0.3    # Poisson's ratio

##   2D Approximation
model = "plane strain"
#model = "plane_stress"

## Alternate Elasticity parameters (dependent on E, nu and 2D approximation)
mu = E/2/(1+nu)
lmbda = E*nu/(1+nu)/(1-2*nu)
if model == "plane_stress":
    lmbda = 2*mu*lmbda/(lmbda+2*mu)

# Define space of functions
#fes = H1(mesh, order=1, dim=mesh.dim)
fes = VectorH1(mesh, order=ele_order)  # Note: Vector H1 space has no. of components = mesh.dim

# Dirichlet boundary conditions
fesx, fesy = fes.components
freedofs = fes.FreeDofs()
tol = 1.0E-14

# Apply Dirichlet BCs (uy=0) on boundary marked by "Ysym"
for el in fesx.Elements(BND):
    if el.mat == "Ysym":
        for dof in el.dofs:
            #print(dof)
            freedofs.Clear(fesx.ndof+dof)


for v in mesh.vertices:
     
    #Apply ux=0 at the bottom-right corner node
    if v.point[0] > (0.5*SB-tol) and v.point[1] < tol:
         for dof in fesx.GetDofNrs(v):
             freedofs.Clear(dof)
#             print(v.point[0], v.point[1], fesx.GetDofNrs(v))

#print(fes.FreeDofs())

u  = fes.TrialFunction()
v  = fes.TestFunction()

force = CoefficientFunction( (0.0,100.0) )
ud = CoefficientFunction(0.0)

#ds = dx(definedon=mesh.Boundaries("Pres_load"))

# some utils ...
def IdentityCF(dim):
    return CoefficientFunction( tuple( [1 if i==j else 0 for i in range(dim) for j in range(dim)]), dims=(dim,dim) )

def Trace(mat):
    return sum( [mat[i,i] for i in range(mat.dims[0]) ])

# Define Strain Tensor
def eps(u):
    return 0.5*(u.Deriv() + u.Deriv().trans)  # NOTE: row .. component, col .. derivative

# Define Stress Tensor
def sigma(u):
    return lmbda*Trace(eps(u))*IdentityCF(2) + 2.0*mu*eps(u)

# the right hand side
f = LinearForm(fes)
#f += SymbolicLFI(force*v, definedon=mesh.Boundaries("Pres_load"))
f += SymbolicLFI(InnerProduct(force,v), definedon=mesh.Boundaries("Pres_load"))
#f += InnerProduct(force, v)*ds

# the bilinear-form 
a = BilinearForm(fes, symmetric=True)
a += SymbolicBFI(InnerProduct(sigma(u), eps(v)))
#a += InnerProduct(sigma(u), eps(v))*dx

a.Assemble()
f.Assemble()

# the solution field 
gfu = GridFunction(fes)

# Obtain solution
gfu.vec.data = a.mat.Inverse(fes.FreeDofs(), inverse="sparsecholesky") * f.vec

fes_sig = L2(mesh, order=ele_order-1, dim=2*mesh.dim)
gf_sig =  GridFunction(fes_sig,"stress")
gf_sig.Set(sigma(gfu))

Draw (gfu, mesh, "displacement")
Draw (gf_sig, mesh, "stress")
SetVisualization (deformation=True)

# VTKOutput object
vtk = VTKOutput(ma=mesh,
                coefs=[gfu, gf_sig],
                names = ["displacement","stresses"],
                filename="SEC_2D_tri_result",
                subdivision=1)
# Exporting the results:
vtk.Do()